/*
 * Access to memory cards.
 *
 * Copyright (C) 2023, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/msc.h>
#include <l4/sys/cache.h>

#include "msc_region_operations.h"



MscRegionOperations::MscRegionOperations(Msc_channel *msc_channel, uint8_t card,
                                         struct dma_region region)
: _msc_channel(msc_channel), _card(card), _region(region)
{
}

long MscRegionOperations::read_partition_table(struct partition_table_entry **entry)
{
  l4_cache_inv_data(_region.vaddr, _region.vaddr + _region.size);

  uint32_t transferred = _msc_channel->read(_card, &_region, 0, _region.size);

  if (!transferred)
    return -L4_EIO;

  *entry = get_partition_table((uint8_t *) _region.vaddr);

  if (*entry == NULL)
    return -L4_EIO;

  return L4_EOK;
}

/* Public operations. */

long MscRegionOperations::get_partition(uint8_t partition,
                                        uint32_t *start_block,
                                        uint32_t *num_blocks)
{
  std::lock_guard<std::mutex> guard(_lock);

  // Use the fileid to select the partition.

  if (partition >= 4)
      return -L4_ENOENT;

  struct partition_table_entry *entry;
  long err = read_partition_table(&entry);

  if (err)
    return err;

  if (!entry[partition].num_sectors)
    return -L4_ENOENT;

  *start_block = entry[partition].lba_first_sector;
  *num_blocks = entry[partition].num_sectors;

  return L4_EOK;
}

void MscRegionOperations::read(l4_addr_t vaddr, l4re_dma_space_dma_addr_t paddr,
                               offset_t filepos, offset_t size)
{
  std::lock_guard<std::mutex> guard(_lock);

  struct dma_region region;

  // NOTE: Truncating the file position and size.

  region.vaddr = vaddr;
  region.paddr = paddr;
  region.size = size;

  l4_cache_inv_data(vaddr, vaddr + size);

  _msc_channel->read(_card, &region, filepos, size);
}

void MscRegionOperations::write(l4_addr_t vaddr, l4re_dma_space_dma_addr_t paddr,
                                offset_t filepos, offset_t size)
{
  std::lock_guard<std::mutex> guard(_lock);

  // NOTE: To be implemented.

  (void) vaddr; (void) paddr; (void) filepos; (void) size;
}
